@@ -1,23 +1,23 @@
 (*
-* Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
-* Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller
-* This file is part of Coccinelle.
-* 
-* Coccinelle is free software: you can redistribute it and/or modify
-* it under the terms of the GNU General Public License as published by
-* the Free Software Foundation, according to version 2 of the License.
-* 
-* Coccinelle is distributed in the hope that it will be useful,
-* but WITHOUT ANY WARRANTY; without even the implied warranty of
-* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-* GNU General Public License for more details.
-* 
-* You should have received a copy of the GNU General Public License
-* along with Coccinelle.  If not, see <http://www.gnu.org/licenses/>.
-* 
-* The authors reserve the right to distribute this or future versions of
-* Coccinelle under other licenses.
-*)
+ * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
+ * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
+ * This file is part of Coccinelle.
+ *
+ * Coccinelle is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, according to version 2 of the License.
+ *
+ * Coccinelle is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with Coccinelle.  If not, see <http://www.gnu.org/licenses/>.
+ *
+ * The authors reserve the right to distribute this or future versions of
+ * Coccinelle under other licenses.
+ *)
 
 
 open Ograph_extended
@@ -30,7 +30,7 @@ type mvar = Ast_cocci.meta_name
 type predicate =
     InLoop | TrueBranch | FalseBranch
   | After (* pointer to the code after an if or while *)
-  | FallThrough
+  | FallThrough | LoopFallThrough
   | Return (* any exit from the current function *)
   | FunHeader | Top | Exit | ErrorExit | Goto
   | Paren of Ast_cocci.meta_name
@@ -55,9 +55,11 @@ type metavars_binding = Ast_c.metavars_binding
 type metavar_binding_kind2 = 
   | NormalMetaVal of Ast_c.metavar_binding_kind
   | ParenVal of Ast_cocci.meta_name
-  | LabelVal of int list
+  | LabelVal of labelval
   | GoodVal | BadVal (* used to implement \+ *)
 
+and labelval = Absolute of int list | Prefix of int list
+
 and metavars_binding2 = (mvar, metavar_binding_kind2) Common.assoc
 
 
