@@ -1,23 +1,23 @@
 (*
-* Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
-* Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller
-* This file is part of Coccinelle.
-* 
-* Coccinelle is free software: you can redistribute it and/or modify
-* it under the terms of the GNU General Public License as published by
-* the Free Software Foundation, according to version 2 of the License.
-* 
-* Coccinelle is distributed in the hope that it will be useful,
-* but WITHOUT ANY WARRANTY; without even the implied warranty of
-* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-* GNU General Public License for more details.
-* 
-* You should have received a copy of the GNU General Public License
-* along with Coccinelle.  If not, see <http://www.gnu.org/licenses/>.
-* 
-* The authors reserve the right to distribute this or future versions of
-* Coccinelle under other licenses.
-*)
+ * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
+ * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
+ * This file is part of Coccinelle.
+ *
+ * Coccinelle is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, according to version 2 of the License.
+ *
+ * Coccinelle is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with Coccinelle.  If not, see <http://www.gnu.org/licenses/>.
+ *
+ * The authors reserve the right to distribute this or future versions of
+ * Coccinelle under other licenses.
+ *)
 
 
 open Common.Infix
@@ -71,13 +71,20 @@ and pp_binding subst =
 let pp_binding_kind2 = function
   | ParenVal s -> pp "pv("; pp_meta s; pp ")"
   | NormalMetaVal x -> pp_binding_kind x
-  | LabelVal xs -> 
+  | LabelVal (Absolute xs) -> 
       begin
         pp "labelval";
         pp "(";
         Common.print_between (fun () -> pp ",") Format.print_int xs;
         pp ")";
       end
+  | LabelVal (Prefix xs) -> 
+      begin
+        pp "prefixlabelval";
+        pp "(";
+        Common.print_between (fun () -> pp ",") Format.print_int xs;
+        pp ")";
+      end
   | GoodVal -> pp "goodval"
   | BadVal ->  pp "badval"
 
@@ -88,6 +95,7 @@ let rec pp_predicate = function
   | FalseBranch -> pp "FalseBranch"
   | After -> pp "After"
   | FallThrough -> pp "FallThrough"
+  | LoopFallThrough -> pp "LoopFallThrough"
   | Return -> pp "Return"
   | FunHeader -> pp "FunHeader"
   | Top -> pp "Top"
