@@ -1,23 +1,23 @@
 (*
-* Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
-* Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller
-* This file is part of Coccinelle.
-* 
-* Coccinelle is free software: you can redistribute it and/or modify
-* it under the terms of the GNU General Public License as published by
-* the Free Software Foundation, according to version 2 of the License.
-* 
-* Coccinelle is distributed in the hope that it will be useful,
-* but WITHOUT ANY WARRANTY; without even the implied warranty of
-* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-* GNU General Public License for more details.
-* 
-* You should have received a copy of the GNU General Public License
-* along with Coccinelle.  If not, see <http://www.gnu.org/licenses/>.
-* 
-* The authors reserve the right to distribute this or future versions of
-* Coccinelle under other licenses.
-*)
+ * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
+ * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
+ * This file is part of Coccinelle.
+ *
+ * Coccinelle is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, according to version 2 of the License.
+ *
+ * Coccinelle is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with Coccinelle.  If not, see <http://www.gnu.org/licenses/>.
+ *
+ * The authors reserve the right to distribute this or future versions of
+ * Coccinelle under other licenses.
+ *)
 
 
 open Common
@@ -259,8 +259,10 @@ let short_options = [
   "-dir", Arg.Set dir,
   "    <dir> process all files in directory recursively";
 
-  "-use_glimpse", Arg.Set Flag.use_glimpse,
+  "-use_glimpse", Arg.Unit (function _ -> Flag.scanner := Flag.Glimpse),
   "    works with -dir, use info generated by glimpseindex";
+  "-use_google", Arg.String (function s -> Flag.scanner := Flag.Google s),
+  "    find relevant files using google code search";
   "-patch", Arg.String (function s -> Flag.patch := Some s),
   ("    <dir> path name with respect to which a patch should be created\n"^
    "    \"\" for a file in the current directory");
@@ -271,13 +273,14 @@ let short_options = [
 
 
   "-version",   Arg.Unit (fun () ->
-    pr2 (spf "spatch version: %s" Config.version);
+    let withpython = if Pycocci.python_support then "with" else "without" in
+    pr2 (spf "spatch version %s %s Python support" Config.version withpython);
     exit 0;
   ),
     "  guess what";
 
   "-date",   Arg.Unit (fun () ->
-    pr2 "version: $Date: 2009/07/28 14:05:49 $";
+    pr2 "version: $Date: 2010/01/04 11:16:30 $";
     raise (Common.UnixExit 0)
     ),
   "   guess what";
@@ -436,9 +439,8 @@ let other_options = [
 
 
   "change of algorithm options",
-  "", 
-  [  
-(* no popl in official version
+  "",
+  [
     "-popl", Arg.Set FC.popl,
     "    simplified SmPL, for the popl paper";
 
@@ -451,16 +453,20 @@ let other_options = [
     Arg.Unit
     (function _ -> FC.popl := true; Flag_popl.keep_all_wits := true),
     "    simplified SmPL, for the popl paper";
-*)
 
     "-hrule", Arg.String
     (function s ->
       Flag.make_hrule := Some s; FC.include_options := FC.I_NO_INCLUDES),
     "    semantic patch generation";
 
+    "-keep_comments", Arg.Set Flag_parsing_c.keep_comments,
+    "   keep comments around removed code";
+
     "-loop",              Arg.Set Flag_ctl.loop_in_src_code,    " ";
     "-no_loops",          Arg.Set Flag_parsing_c.no_loops,
     "   drop all back edges derived from looping constructs - unsafe";
+    "-no_gotos",          Arg.Set Flag_parsing_c.no_gotos,
+    "   drop all jumps derived from gotos - unsafe";
 
     "-l1",                Arg.Clear Flag_parsing_c.label_strategy_2, " ";
     "-ifdef_to_if",       Arg.Set FC.ifdef_to_if,
@@ -477,7 +483,7 @@ let other_options = [
 
 
     "-disallow_nested_exps", Arg.Set Flag_matcher.disallow_nested_exps,
-       "disallow an expresion pattern from matching a term and its subterm";
+       " disallow an expresion pattern from matching a term and its subterm";
     "-disable_worth_trying_opt", Arg.Clear FC.worth_trying_opt,
     "  ";
     "-only_return_is_error_exit",
@@ -500,6 +506,8 @@ let other_options = [
     "  spacing of + code follows the conventions of Linux";
     "-smpl_spacing", Arg.Unit Flag_parsing_c.set_smpl_spacing,
     "  spacing of + code follows the semantic patch";
+    "-D", Arg.String Flag_parsing_cocci.set_defined_virtual_rules,
+    "  indicate that a virtual rule should be considered to be matched";
   ];
 
   "misc options",
@@ -635,8 +643,8 @@ let glimpse_filter (coccifile, isofile) dir =
     Cocci.sp_of_file coccifile (Some isofile) in
   match query with
     None -> pr2 "no glimpse keyword inferred from snippet"; None
-  | Some query ->
-      let suffixes = if !include_headers then ["c";"h"] else ["c"] in
+  | Some [query] ->
+      (let suffixes = if !include_headers then ["c";"h"] else ["c"] in
       pr2 ("glimpse request = " ^ query);
       let command = spf "glimpse -y -H %s -N -W -w '%s'" dir query in
       let (glimpse_res,stat) = Common.cmd_to_list_and_status command in
@@ -646,8 +654,8 @@ let glimpse_filter (coccifile, isofile) dir =
 	    (glimpse_res +>
 	     List.filter
 	       (fun file -> List.mem (Common.filesuffix file) suffixes))
-      |	_ -> None (* error, eg due to pattern too big *)
-
+      |	_ -> None (* error, eg due to pattern too big *))
+  | _ -> failwith "not possible"
 
 (*****************************************************************************)
 (* Main action *)
@@ -681,7 +689,7 @@ let main_action xs =
 
         let infiles =
           Common.profile_code "Main.infiles computation" (fun () ->
-	    match !dir, !kbuild_info, !Flag.use_glimpse with
+	    match !dir, !kbuild_info, !Flag.scanner = Flag.Glimpse with
             (* glimpse *)
             | false, _, true ->
                 failwith "-use_glimpse works only with -dir"
@@ -769,6 +777,7 @@ let main_action xs =
 		pr2 ("HANDLING: " ^ (join " " cfiles));
 		Common.timeout_function_opt !FC.timeout (fun () ->
   	          Common.report_if_take_time 10 (join " " cfiles) (fun () ->
+		    (*let s = profile_diagnostic() in*)
                     (* Unix.sleep 1; *)
                     try
 		      let optfile =
@@ -784,6 +793,10 @@ let main_action xs =
 		    with
 		      | Common.UnixExit x -> raise (Common.UnixExit x)
 		      | e ->
+			  (*pr2 "previous";
+			  pr2 s;
+			  pr2 "new";
+			  pr2(profile_diagnostic());*)
 			  if !dir
 			  then begin
 			    pr2 ("EXN:" ^ Printexc.to_string e);
@@ -946,8 +959,7 @@ let main () =
     (* --------------------------------------------------------- *)
     (* This is the main entry *)
     (* --------------------------------------------------------- *)
-    | x::xs ->
-        main_action (x::xs)
+    | x::xs -> main_action (x::xs)
 
     (* --------------------------------------------------------- *)
     (* empty entry *)
