@@ -296,7 +296,7 @@ let rec (aux_statement: (nodei option * xinfo) -> statement -> nodei option) =
   match Ast_c.unwrap_st stmt with
 
   (*  coupling: the Switch case copy paste parts of the Compound case *)
-  | Ast_c.Compound statxs -> 
+  | Ast_c.Compound statxs ->
       (* flow_to_ast: *)
       let (i1, i2) = tuple_of_list2 ii in
 
@@ -331,14 +331,14 @@ let rec (aux_statement: (nodei option * xinfo) -> statement -> nodei option) =
       in
 
       !g +> add_arc_opt (starti, newi);
-      let starti = Some newi in
+      let finishi = Some newi in
 
-      aux_statement_list starti (xi, newxi) statxs
+      aux_statement_list finishi (xi, newxi) statxs
 
       (* braces: *)
-      +> Common.fmap (fun starti -> 
+      +> Common.fmap (fun finishi -> 
             (* subtil: not always return a Some.
-             * Note that if starti is None, alors forcement ca veut dire
+             * Note that if finishi is None, alors forcement ca veut dire
              * qu'il y'a eu un return (ou goto), et donc forcement les 
              * braces auront au moins ete crée une fois, et donc flow_to_ast
              * marchera.
@@ -347,7 +347,12 @@ let rec (aux_statement: (nodei option * xinfo) -> statement -> nodei option) =
              * il faut forcement au moins un return.
              *)
             let endi = !g#add_node endnode in
-            !g#add_arc ((starti, endi), Direct);
+	    if xi.compound_caller = Statement
+	    then
+	      (let afteri = !g +> add_node AfterNode lbl "[after]" in
+	      !g#add_arc ((newi, afteri), Direct);
+	      !g#add_arc ((afteri, endi), Direct));
+            !g#add_arc ((finishi, endi), Direct);
             endi 
            ) 
 
@@ -369,23 +374,28 @@ let rec (aux_statement: (nodei option * xinfo) -> statement -> nodei option) =
      in
      !g +> add_arc_opt (starti, newi);
 
-     let ilabel = 
-       try xi.labels_assoc#find s 
-       with Not_found -> 
+     if !Flag_parsing_c.no_gotos
+     then Some newi
+     else
+       begin
+	 let ilabel = 
+	   try xi.labels_assoc#find s 
+	   with Not_found -> 
          (* jump vers ErrorExit a la place ? 
           * pourquoi tant de "cant jump" ? pas detecté par gcc ? 
           *)
-         raise (Error (GotoCantFindLabel (s, pinfo_of_ii ii)))
-     in
-     (* !g +> add_arc_opt (starti, ilabel); 
-      * todo: special_case: suppose that always goto to toplevel of function,
-      * hence the Common.init 
-      * todo?: can perhaps report when a goto is not a classic error_goto ? 
-      * that is when it does not jump to the toplevel of the function.
-      *)
-     let newi = insert_all_braces (Common.list_init xi.braces) newi in
-     !g#add_arc ((newi, ilabel), Direct);
-     None
+             raise (Error (GotoCantFindLabel (s, pinfo_of_ii ii)))
+	 in
+	 (* !g +> add_arc_opt (starti, ilabel); 
+	  * todo: special_case: suppose that always goto to toplevel of
+	  * function, hence the Common.init 
+	  * todo?: can perhaps report when a goto is not a classic error_goto ?
+	  * that is when it does not jump to the toplevel of the function.
+	  *)
+	 let newi = insert_all_braces (Common.list_init xi.braces) newi in
+	 !g#add_arc ((newi, ilabel), Direct);
+	 None
+       end
       
   | Jump (Ast_c.GotoComputed e) -> 
       raise (Error (ComputedGoto))
@@ -431,7 +441,7 @@ let rec (aux_statement: (nodei option * xinfo) -> statement -> nodei option) =
     let iist2 = Ast_c.get_ii_st_take_care st2 in
     (match Ast_c.unwrap_st st2 with
     | Ast_c.ExprStatement (None) when null iist2 -> 
-      (* sometome can have ExprStatement None but it is a if-then-else,
+      (* sometime can have ExprStatement None but it is a if-then-else,
        * because something like   if() xx else ;
        * so must force to have [] in the ii associated with ExprStatement 
        *)
@@ -570,7 +580,7 @@ let rec (aux_statement: (nodei option * xinfo) -> statement -> nodei option) =
                end;
                newxi'
              in
-             let newxi = { xi with compound_caller = 
+             let newxi = { xi_lbl with compound_caller = (* was xi *)
                  Switch todo_in_compound 
              } 
              in
@@ -700,7 +710,7 @@ let rec (aux_statement: (nodei option * xinfo) -> statement -> nodei option) =
       !g +> add_arc_opt (starti, newi);
       let newfakethen = !g +> add_node InLoopNode  lbl "[whiletrue]" in
       (* let newfakeelse = !g +> add_node FalseNode lbl "[endwhile]" in *)
-      let newafter = !g +> add_node FallThroughNode lbl "[whilefall]" in
+      let newafter = !g +> add_node LoopFallThroughNode lbl "[whilefall]" in
       let newfakeelse = 
         !g +> add_node (EndStatement (Some iifakeend)) lbl "[endwhile]" in
 
@@ -749,13 +759,17 @@ let rec (aux_statement: (nodei option * xinfo) -> statement -> nodei option) =
       let newfakeelse = 
         !g +> add_node (EndStatement (Some iifakeend)) lbl "[enddowhile]" in
 
+      let afteri = !g +> add_node AfterNode lbl "[after]" in
+      !g#add_arc ((doi,afteri), Direct);
+      !g#add_arc ((afteri,newfakeelse), Direct);
+
       let newxi = { xi_lbl with
          ctx = LoopInfo (taili, newfakeelse, xi_lbl.braces, lbl);
          ctx_stack = xi_lbl.ctx::xi_lbl.ctx_stack
         }
       in
 
-      if not is_zero && not !Flag_parsing_c.no_loops
+      if not is_zero && (not !Flag_parsing_c.no_loops)
       then begin
         let newfakethen = !g +> add_node InLoopNode lbl "[dowhiletrue]" in
         !g#add_arc ((taili, newfakethen), Direct); 
@@ -788,7 +802,7 @@ let rec (aux_statement: (nodei option * xinfo) -> statement -> nodei option) =
       !g +> add_arc_opt (starti, newi);
       let newfakethen = !g +> add_node InLoopNode  lbl "[fortrue]" in
       (*let newfakeelse = !g +> add_node FalseNode lbl "[endfor]" in*)
-      let newafter = !g +> add_node FallThroughNode lbl "[forfall]" in
+      let newafter = !g +> add_node LoopFallThroughNode lbl "[forfall]" in
       let newfakeelse = 
         !g +> add_node (EndStatement (Some iifakeend)) lbl "[endfor]" in
 
@@ -803,7 +817,8 @@ let rec (aux_statement: (nodei option * xinfo) -> statement -> nodei option) =
       !g#add_arc ((newi, newafter), Direct);
       let finalthen = aux_statement (Some newfakethen, newxi) st in
       !g +> add_arc_opt
-	(finalthen, if !Flag_parsing_c.no_loops then newafter else newi);
+	(finalthen,
+	 if !Flag_parsing_c.no_loops then newafter else newi);
       Some newfakeelse
 
 
@@ -824,7 +839,7 @@ let rec (aux_statement: (nodei option * xinfo) -> statement -> nodei option) =
       !g +> add_arc_opt (starti, newi);
       let newfakethen = !g +> add_node InLoopNode  lbl "[fortrue]" in
       (*let newfakeelse = !g +> add_node FalseNode lbl "[endfor]" in*)
-      let newafter = !g +> add_node FallThroughNode lbl "[foreachfall]" in
+      let newafter = !g +> add_node LoopFallThroughNode lbl "[foreachfall]" in
       let newfakeelse = 
         !g +> add_node (EndStatement (Some iifakeend)) lbl "[endforeach]" in
 
@@ -839,7 +854,8 @@ let rec (aux_statement: (nodei option * xinfo) -> statement -> nodei option) =
       !g#add_arc ((newi, newafter), Direct);
       let finalthen = aux_statement (Some newfakethen, newxi) st in
       !g +> add_arc_opt
-	(finalthen, if !Flag_parsing_c.no_loops then newafter else newi);
+	(finalthen,
+	 if !Flag_parsing_c.no_loops then newafter else newi);
       Some newfakeelse
 
 
@@ -893,7 +909,10 @@ let rec (aux_statement: (nodei option * xinfo) -> statement -> nodei option) =
           let desti = 
             (match x with 
             | Ast_c.Break -> loopendi 
-            | Ast_c.Continue -> loopstarti 
+            | Ast_c.Continue ->
+		(* if no loops, then continue behaves like break - just
+		   one iteration *)
+		if !Flag_parsing_c.no_loops then loopendi else loopstarti 
             | x -> raise Impossible
             ) in
           let difference = List.length xi.braces - List.length braces in
@@ -1030,6 +1049,17 @@ and aux_statement_list starti (xi, newxi) statxs =
             !g +> add_arc_opt (finalthen, taili);
           ) 
         in
+
+(*
+        This is an attempt to let a statement metavariable match this
+	construct, but it doesn't work because #ifdef is not a statement.
+        Not sure if this is a good or bad thing, at least if there is no else
+	because then no statement might be there.
+	let afteri = !g +> add_node AfterNode newxi'.labels "[after]" in
+	!g#add_arc ((newi, afteri), Direct);
+	!g#add_arc ((afteri, taili), Direct);
+*)
+
         Some taili
 
   ) starti
@@ -1116,7 +1146,7 @@ let specialdeclmacro_to_stmt (s, args, ii) =
 
 
 
-let ast_to_control_flow e = 
+let ast_to_control_flow e =
 
   (* globals (re)initialialisation *) 
   g := (new ograph_mutable);
@@ -1260,8 +1290,7 @@ let annotate_loop_nodes g =
         let node = g#nodes#find yi in
         let ((node2, nodeinfo), nodestr) = node in
         let node' = ((node2, {nodeinfo with is_loop = true}), (nodestr ^ "*")) 
-        in
-        g#replace_node (yi, node');
+        in g#replace_node (yi, node');
     );
   );
 
