@@ -3,18 +3,18 @@
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License (GPL)
  * version 2 as published by the Free Software Foundation.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * file license.txt for more details.
- * 
+ *
  * This file was part of Coccinelle.
  *)
 open Common
 
 (*****************************************************************************)
-(* mostly a copy paste of parsing_cocci/pretty_print_cocci.ml 
+(* mostly a copy paste of parsing_cocci/pretty_print_cocci.ml
  * todo?: try to factorize ?
  *)
 (*****************************************************************************)
@@ -23,8 +23,8 @@ module Ast = Ast_cocci
 
 let term s = Ast.unwrap_mcode s
 
-(* or perhaps can have in plus, for instance a Disj, but those Disj must be 
- *  handled by interactive tool (by proposing alternatives) 
+(* or perhaps can have in plus, for instance a Disj, but those Disj must be
+ *  handled by interactive tool (by proposing alternatives)
  *)
 exception CantBeInPlus
 
@@ -34,7 +34,7 @@ type pos = Before | After | InPlace
 
 let unknown = -1
 
-let rec pp_list_list_any
+let rec do_all
     (env, pr, pr_celem, pr_cspace, pr_space, pr_arity, pr_barrier,
      indent, unindent)
     generating xxs before =
@@ -45,7 +45,7 @@ let print_string s line lcol =
   pr s line lcol rcol in
 let print_text s = pr s unknown unknown unknown in
 let close_box _ = () in
-let force_newline () = print_text "\n" in
+let force_newline _ = print_text "\n" in
 
 let start_block () = force_newline(); indent() in
 let end_block () = unindent(); force_newline () in
@@ -98,9 +98,9 @@ and print_anything_list = function
 
 let print_around printer term = function
     Ast.NOTHING -> printer term
-  | Ast.BEFORE(bef) -> print_anything bef; printer term
-  | Ast.AFTER(aft) -> printer term; print_anything aft
-  | Ast.BEFOREAFTER(bef,aft) ->
+  | Ast.BEFORE(bef,_) -> print_anything bef; printer term
+  | Ast.AFTER(aft,_) -> printer term; print_anything aft
+  | Ast.BEFOREAFTER(bef,aft,_) ->
       print_anything bef; printer term; print_anything aft in
 
 let print_string_befaft fn fn1 x info =
@@ -111,7 +111,6 @@ let print_string_befaft fn fn1 x info =
   List.iter
     (function (s,ln,col) -> force_newline(); fn1(); print_string s ln col)
     info.Ast.straft in
-
 let print_meta (r,x) = print_text x in
 
 let print_pos = function
@@ -146,6 +145,14 @@ let mcode fn (s,info,mc,pos) =
       |	_ -> force_newline());
       fn s line lcol;
       let _ = print_comments (Some info.Ast.line) info.Ast.straft in
+      (* newline after a pragma
+	 should really store parsed versions of the strings, but make a cheap
+	 effort here
+         print_comments takes care of interior newlines *)
+      (match List.rev info.Ast.straft with
+	(str,_,_)::_ when String.length str > 0 && String.get str 0 = '#' ->
+	  force_newline()
+      |	_ -> ());
       ()
       (* printing for rule generation *)
   | (true, Ast.MINUS(_,_,_,plus_stream)) ->
@@ -156,10 +163,14 @@ let mcode fn (s,info,mc,pos) =
   | (true, Ast.CONTEXT(_,plus_streams)) ->
       let fn s = force_newline(); fn s line lcol; print_pos pos in
       print_around fn s plus_streams
-  | (true,Ast.PLUS) ->
+  | (true,Ast.PLUS Ast.ONE) ->
       let fn s =
 	force_newline(); print_text "+ "; fn s line lcol; print_pos pos in
       print_string_befaft fn (function _ -> print_text "+ ") s info
+  | (true,Ast.PLUS Ast.MANY) ->
+      let fn s =
+ 	force_newline(); print_text "++ "; fn s line lcol; print_pos pos in
+      print_string_befaft fn (function _ -> print_text "++ ") s info
 in
 
 
@@ -220,25 +231,25 @@ in
 
 let rec ident i =
   match Ast.unwrap i with
-    Ast.Id(name) -> mcode print_string name
-  | Ast.MetaId(name,_,_,_) -> 
-      handle_metavar name (function
-        | (Ast_c.MetaIdVal id) -> print_text id
-        | _ -> raise Impossible
-        ) 
-  | Ast.MetaFunc(name,_,_,_) -> 
-      handle_metavar name (function
-        | (Ast_c.MetaFuncVal id) -> print_text id
-        | _ -> raise Impossible
-        ) 
-  | Ast.MetaLocalFunc(name,_,_,_) -> 
-      handle_metavar name (function
-        | (Ast_c.MetaLocalFuncVal id) -> print_text id
-        | _ -> raise Impossible
-        )
+      Ast.Id(name) -> mcode print_string name
+    | Ast.MetaId(name,_,_,_) ->
+	handle_metavar name (function
+			       | (Ast_c.MetaIdVal id) -> print_text id
+			       | _ -> raise Impossible
+			    )
+    | Ast.MetaFunc(name,_,_,_) ->
+	handle_metavar name (function
+			       | (Ast_c.MetaFuncVal id) -> print_text id
+			       | _ -> raise Impossible
+			    )
+    | Ast.MetaLocalFunc(name,_,_,_) ->
+	handle_metavar name (function
+			       | (Ast_c.MetaLocalFuncVal id) -> print_text id
+			       | _ -> raise Impossible
+			    )
 
-  | Ast.OptIdent(_) | Ast.UniqueIdent(_) -> 
-      raise CantBeInPlus
+    | Ast.OptIdent(_) | Ast.UniqueIdent(_) ->
+	raise CantBeInPlus
 
 in
 
@@ -256,7 +267,12 @@ let rec expression e =
   | Ast.Constant(const) -> mcode constant const
   | Ast.FunCall(fn,lp,args,rp) ->
       expression fn; mcode print_string_box lp;
-      dots (function _ -> ()) expression args;
+      let comma e =
+	expression e;
+	match Ast.unwrap e with
+	  Ast.EComma(cm) -> pr_space()
+	| _ -> () in
+      dots (function _ -> ()) comma args;
       close_box(); mcode print_string rp
   | Ast.Assignment(left,op,right,_) ->
       expression left; pr_space(); mcode assignOp op;
@@ -608,7 +624,13 @@ and parameterTypeDef p =
   | Ast.Pdots(dots) | Ast.Pcircles(dots) -> raise CantBeInPlus
   | Ast.OptParam(param) | Ast.UniqueParam(param) -> raise CantBeInPlus
 
-and parameter_list l = dots (function _ -> ()) parameterTypeDef l
+and parameter_list l =
+  let comma p =
+    parameterTypeDef p;
+    match Ast.unwrap p with
+      Ast.PComma(cm) -> pr_space()
+    | _ -> () in
+  dots (function _ -> ()) comma l
 in
 
 
@@ -791,8 +813,9 @@ let rec statement arity s =
       indent_if_needed body (function _ -> statement arity body);
       mcode (fun _ _ _ -> ()) ((),Ast.no_info,aft,Ast.NoMetaPos)
 
-  | Ast.Switch(header,lb,cases,rb) ->
+  | Ast.Switch(header,lb,decls,cases,rb) ->
       rule_elem arity header; pr_space(); rule_elem arity lb;
+      dots force_newline (statement arity) decls;
       List.iter (function x -> case_line arity x; force_newline()) cases;
       rule_elem arity rb
 
@@ -1030,3 +1053,12 @@ in
       (* print a newline at the end, if needed *)
       newline_after()
 
+let rec pp_list_list_any (envs, pr, pr_celem, pr_cspace, pr_space, pr_arity,
+			  pr_barrier, indent, unindent)
+    generating xxs before =
+  List.iter
+    (function env ->
+      do_all (env, pr, pr_celem, pr_cspace, pr_space, pr_arity, pr_barrier,
+	      indent, unindent)
+	generating xxs before)
+    envs
