@@ -1,25 +1,32 @@
 (*
-* Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
-* Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller
-* This file is part of Coccinelle.
-* 
-* Coccinelle is free software: you can redistribute it and/or modify
-* it under the terms of the GNU General Public License as published by
-* the Free Software Foundation, according to version 2 of the License.
-* 
-* Coccinelle is distributed in the hope that it will be useful,
-* but WITHOUT ANY WARRANTY; without even the implied warranty of
-* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-* GNU General Public License for more details.
-* 
-* You should have received a copy of the GNU General Public License
-* along with Coccinelle.  If not, see <http://www.gnu.org/licenses/>.
-* 
-* The authors reserve the right to distribute this or future versions of
-* Coccinelle under other licenses.
-*)
+ * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
+ * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
+ * This file is part of Coccinelle.
+ *
+ * Coccinelle is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, according to version 2 of the License.
+ *
+ * Coccinelle is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with Coccinelle.  If not, see <http://www.gnu.org/licenses/>.
+ *
+ * The authors reserve the right to distribute this or future versions of
+ * Coccinelle under other licenses.
+ *)
 
 
+(* Constraints on Meta-* Identifiers, Functions *)
+type idconstraint =
+    IdNoConstraint
+  | IdNegIdSet      of string list
+  | IdRegExp        of string * Str.regexp
+  | IdNotRegExp     of string * Str.regexp
+
 (* --------------------------------------------------------------------- *)
 (* Modified code *)
 
@@ -46,24 +53,33 @@ type 'a wrap =
       iso_info : (string*anything) list }
 
 and 'a befaft =
-    BEFORE      of 'a list list
-  | AFTER       of 'a list list
-  | BEFOREAFTER of 'a list list * 'a list list
+    BEFORE      of 'a list list * count
+  | AFTER       of 'a list list * count
+  | BEFOREAFTER of 'a list list * 'a list list * count
   | NOTHING
 
 and 'a mcode = 'a * info * mcodekind * meta_pos (* pos variable *)
- (* pos is an offset indicating where in the C code the mcodekind has an
- effect *)
-(* int list is the match instances, which are only meaningful in annotated
-C code *)
-(* int is the adjacency index, which is incremented on context dots *)
- and mcodekind =
+    (* pos is an offset indicating where in the C code the mcodekind
+       has an effect *)
+    (* int list is the match instances, which are only meaningful in annotated
+       C code *)
+    (* int is the adjacency index, which is incremented on context dots *)
+(* iteration is only allowed on contect code, the intuition vaguely being
+that there is no way to replace something more than once.  Actually,
+allowing iterated additions on minus code would cause problems with some
+heuristics for adding braces, because one couldn't identify simple
+replacements with certainty.  Anyway, iteration doesn't seem to be needed
+on - code for the moment.  Although it may be confusing that there can be
+iterated addition of code before context code where the context code is
+immediately followed by removed code. *)
+and mcodekind =
     MINUS       of pos * int list * int * anything list list
   | CONTEXT     of pos * anything befaft
-  | PLUS
- and fixpos =
+  | PLUS        of count
+and count = ONE (* + *) | MANY (* ++ *)
+and fixpos =
     Real of int (* charpos *) | Virt of int * int (* charpos + offset *)
- and pos = NoPos | DontCarePos | FixPos of (fixpos * fixpos)
+and pos = NoPos | DontCarePos | FixPos of (fixpos * fixpos)
 
 and dots_bef_aft =
     NoDots
@@ -127,11 +143,10 @@ and 'a dots = 'a base_dots wrap
 (* Identifier *)
 
 and base_ident =
-    Id of string mcode
-
-  | MetaId        of meta_name mcode * ident list * keep_binding * inherited
-  | MetaFunc      of meta_name mcode * ident list * keep_binding * inherited
-  | MetaLocalFunc of meta_name mcode * ident list * keep_binding * inherited
+    Id            of string mcode
+  | MetaId        of meta_name mcode * idconstraint * keep_binding * inherited
+  | MetaFunc      of meta_name mcode * idconstraint * keep_binding * inherited
+  | MetaLocalFunc of meta_name mcode * idconstraint * keep_binding * inherited
 
   | OptIdent      of ident
   | UniqueIdent   of ident
@@ -170,9 +185,9 @@ and base_expression =
   | Paren          of string mcode (* ( *) * expression *
                       string mcode (* ) *)
 
-  | MetaErr        of meta_name mcode * expression list * keep_binding *
+  | MetaErr        of meta_name mcode * constraints * keep_binding *
 	              inherited
-  | MetaExpr       of meta_name mcode * expression list * keep_binding *
+  | MetaExpr       of meta_name mcode * constraints * keep_binding *
 	              Type_cocci.typeC list option * form * inherited
   | MetaExprList   of meta_name mcode * listlen option * keep_binding *
                       inherited (* only in arg lists *)
@@ -192,6 +207,11 @@ and base_expression =
   | OptExp         of expression
   | UniqueExp      of expression
 
+and constraints =
+    NoConstraint
+  | NotIdCstrt     of idconstraint
+  | NotExpCstrt    of expression list
+
 (* ANY = int E; ID = idexpression int X; CONST = constant int X; *)
 and form = ANY | ID | LocalID | CONST (* form for MetaExp *)
 
@@ -449,7 +469,7 @@ and base_statement =
   | For           of rule_elem (* header *) * statement * end_info (*endfor*)
   | Iterator      of rule_elem (* header *) * statement * end_info (*enditer*)
   | Switch        of rule_elem (* header *) * rule_elem (* { *) *
-	             case_line list * rule_elem (* } *)
+	             statement (*decl*) dots * case_line list * rule_elem(*}*)
   | Atomic        of rule_elem
   | Disj          of statement dots list
   | Nest          of statement dots *
@@ -543,7 +563,7 @@ and dependency =
   | NeverDep of string (* rule never applies for any binding *)
   | AndDep of dependency * dependency
   | OrDep of dependency * dependency
-  | NoDep
+  | NoDep | FailDep
 
 and rule_with_metavars = metavar list * rule
 
@@ -592,6 +612,13 @@ let mkToken x = Token (x,None)
 
 (* --------------------------------------------------------------------- *)
 
+let lub_count i1 i2 =
+  match (i1,i2) with
+    (MANY,MANY) -> MANY
+  | _ -> ONE
+
+(* --------------------------------------------------------------------- *)
+
 let rewrap model x         = {model with node = x}
 let rewrap_mcode (_,a,b,c) x = (x,a,b,c)
 let unwrap x               = x.node
