@@ -1,23 +1,23 @@
 (*
-* Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
-* Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller
-* This file is part of Coccinelle.
-* 
-* Coccinelle is free software: you can redistribute it and/or modify
-* it under the terms of the GNU General Public License as published by
-* the Free Software Foundation, according to version 2 of the License.
-* 
-* Coccinelle is distributed in the hope that it will be useful,
-* but WITHOUT ANY WARRANTY; without even the implied warranty of
-* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-* GNU General Public License for more details.
-* 
-* You should have received a copy of the GNU General Public License
-* along with Coccinelle.  If not, see <http://www.gnu.org/licenses/>.
-* 
-* The authors reserve the right to distribute this or future versions of
-* Coccinelle under other licenses.
-*)
+ * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
+ * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
+ * This file is part of Coccinelle.
+ *
+ * Coccinelle is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, according to version 2 of the License.
+ *
+ * Coccinelle is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with Coccinelle.  If not, see <http://www.gnu.org/licenses/>.
+ *
+ * The authors reserve the right to distribute this or future versions of
+ * Coccinelle under other licenses.
+ *)
 
 
 (* For minus fragment, checks that all of the identifier metavariables that
@@ -83,31 +83,33 @@ let is_ifdef name =
 
 let ident context old_metas table minus i =
   match Ast0.unwrap i with
-    Ast0.Id((name,_,info,_,_,_) : string Ast0.mcode) ->
-      let rl = info.Ast0.pos_info.Ast0.line_start in
-      let err =
-	if List.exists (function x -> x = name) old_metas
-	    && (minus || Ast0.get_mcodekind i = Ast0.PLUS)
-	then
-	  begin
-	    warning
-	      (Printf.sprintf
-		 "line %d: %s, previously declared as a metavariable, is used as an identifier" rl name);
-	      true
-	  end
-	else false in
-      (match context with
-	ID ->
-	  if not (is_ifdef name) && minus && not err(* warn only once per id *)
+      Ast0.Id((name,_,info,_,_,_) : string Ast0.mcode) ->
+	let rl = info.Ast0.pos_info.Ast0.line_start in
+	let is_plus i =
+ 	  match Ast0.get_mcodekind i with Ast0.PLUS _ -> true | _ -> false in
+	let err =
+	  if List.exists (function x -> x = name) old_metas
+	    && (minus || is_plus i)
 	  then
-	    warning
-	      (Printf.sprintf "line %d: should %s be a metavariable?" rl name)
-      | _ -> ())
-  | Ast0.MetaId(name,_,_) -> check_table table minus name
-  | Ast0.MetaFunc(name,_,_) -> check_table table minus name
-  | Ast0.MetaLocalFunc(name,_,_) -> check_table table minus name
-  | Ast0.OptIdent(_) | Ast0.UniqueIdent(_) ->
-      failwith "unexpected code"
+	    begin
+	      warning
+		(Printf.sprintf
+		   "line %d: %s, previously declared as a metavariable, is used as an identifier" rl name);
+	      true
+	    end
+	  else false in
+	  (match context with
+	       ID ->
+		 if not (is_ifdef name) && minus && not err(* warn only once per id *)
+		 then
+		   warning
+		     (Printf.sprintf "line %d: should %s be a metavariable?" rl name)
+	     | _ -> ())
+    | Ast0.MetaId(name,_,_) -> check_table table minus name
+    | Ast0.MetaFunc(name,_,_) -> check_table table minus name
+    | Ast0.MetaLocalFunc(name,_,_) -> check_table table minus name
+    | Ast0.OptIdent(_) | Ast0.UniqueIdent(_) ->
+	failwith "unexpected code"
 
 (* --------------------------------------------------------------------- *)
 (* Expression *)
@@ -334,8 +336,9 @@ and statement old_metas table minus s =
       ident GLOBAL old_metas table minus nm;
       dots (expression ID old_metas table minus) args;
       statement old_metas table minus body
-  | Ast0.Switch(switch,lp,exp,rp,lb,cases,rb) ->
+  | Ast0.Switch(switch,lp,exp,rp,lb,decls,cases,rb) ->
       expression ID old_metas table minus exp;
+      dots (statement old_metas table minus) decls;
       dots (case_line old_metas table minus) cases
   | Ast0.ReturnExpr(ret,exp,sem) -> expression ID old_metas table minus exp
   | Ast0.MetaStmt(name,_) ->     check_table table minus name
@@ -363,13 +366,27 @@ and statement old_metas table minus s =
       parameter_list old_metas table minus params;
       dots (statement old_metas table minus) body
   | Ast0.Include(inc,s) -> () (* no metavariables possible *)
-  | Ast0.Define(def,id,_,body) ->
+  | Ast0.Define(def,id,params,body) ->
       ident GLOBAL old_metas table minus id;
+      define_parameters old_metas table minus params;
       dots (statement old_metas table minus) body
   | Ast0.Label(i,_) -> ident ID old_metas table minus i
   | Ast0.Goto(_,i,_) -> ident ID old_metas table minus i
   | _ -> () (* no metavariable subterms *)
 
+and define_param old_metas table minus p =
+  match Ast0.unwrap p with
+    Ast0.DParam(id) -> ident GLOBAL old_metas table minus id
+  | Ast0.DPComma(_) | Ast0.DPdots(_) | Ast0.DPcircles(_) ->
+      () (* no metavariable subterms *)
+  | Ast0.OptDParam(dp)    -> define_param old_metas table minus dp
+  | Ast0.UniqueDParam(dp) -> define_param old_metas table minus dp
+
+and define_parameters old_metas table minus x =
+  match Ast0.unwrap x with
+    Ast0.NoParams -> ()
+  | Ast0.DParams(lp,dp,rp) -> dots (define_param old_metas table minus) dp
+
 and fninfo old_metas table minus = function
     Ast0.FStorage(stg) -> ()
   | Ast0.FType(ty) -> typeC old_metas table minus ty
@@ -389,6 +406,8 @@ and case_line old_metas table minus c =
       dots (statement old_metas table minus) code
   | Ast0.Case(case,exp,colon,code) ->
       dots (statement old_metas table minus) code
+  | Ast0.DisjCase(_,case_lines,_,_) ->
+      List.iter (case_line old_metas table minus) case_lines
   | Ast0.OptCase(case) -> failwith "unexpected code"
 
 (* --------------------------------------------------------------------- *)
