@@ -1,23 +1,23 @@
 (*
-* Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
-* Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller
-* This file is part of Coccinelle.
-* 
-* Coccinelle is free software: you can redistribute it and/or modify
-* it under the terms of the GNU General Public License as published by
-* the Free Software Foundation, according to version 2 of the License.
-* 
-* Coccinelle is distributed in the hope that it will be useful,
-* but WITHOUT ANY WARRANTY; without even the implied warranty of
-* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-* GNU General Public License for more details.
-* 
-* You should have received a copy of the GNU General Public License
-* along with Coccinelle.  If not, see <http://www.gnu.org/licenses/>.
-* 
-* The authors reserve the right to distribute this or future versions of
-* Coccinelle under other licenses.
-*)
+ * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
+ * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
+ * This file is part of Coccinelle.
+ *
+ * Coccinelle is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, according to version 2 of the License.
+ *
+ * Coccinelle is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with Coccinelle.  If not, see <http://www.gnu.org/licenses/>.
+ *
+ * The authors reserve the right to distribute this or future versions of
+ * Coccinelle under other licenses.
+ *)
 
 
 (* Computes starting and ending logical lines for statements and
@@ -207,15 +207,15 @@ let dots is_dots prev fn d =
 
 let rec ident i =
   match Ast0.unwrap i with
-    Ast0.Id(name) as ui ->
-      let name = promote_mcode name in mkres i ui name name
-  | Ast0.MetaId(name,_,_)
-  | Ast0.MetaFunc(name,_,_) | Ast0.MetaLocalFunc(name,_,_) as ui ->
-      let name = promote_mcode name in mkres i ui name name
-  | Ast0.OptIdent(id) ->
-      let id = ident id in mkres i (Ast0.OptIdent(id)) id id
-  | Ast0.UniqueIdent(id) ->
-      let id = ident id in mkres i (Ast0.UniqueIdent(id)) id id
+      Ast0.Id(name) as ui ->
+	let name = promote_mcode name in mkres i ui name name
+    | Ast0.MetaId(name,_,_)
+    | Ast0.MetaFunc(name,_,_) | Ast0.MetaLocalFunc(name,_,_) as ui ->
+	let name = promote_mcode name in mkres i ui name name
+    | Ast0.OptIdent(id) ->
+	let id = ident id in mkres i (Ast0.OptIdent(id)) id id
+    | Ast0.UniqueIdent(id) ->
+	let id = ident id in mkres i (Ast0.UniqueIdent(id)) id id
 
 (* --------------------------------------------------------------------- *)
 (* Expression *)
@@ -293,7 +293,7 @@ let rec expression e =
   | Ast0.MetaExprList(name,_,_) as ue ->
       let ln = promote_mcode name in mkres e ue ln ln
   | Ast0.EComma(cm) ->
-      let cm = bad_mcode cm in
+      (*let cm = bad_mcode cm in*) (* why was this bad??? *)
       let ln = promote_mcode cm in
       mkres e (Ast0.EComma(cm)) ln ln
   | Ast0.DisjExpr(starter,exps,mids,ender) ->
@@ -540,7 +540,7 @@ and parameterTypeDef p =
   | Ast0.MetaParamList(name,_,_) as up ->
       let ln = promote_mcode name in mkres p up ln ln
   | Ast0.PComma(cm) ->
-      let cm = bad_mcode cm in
+      (*let cm = bad_mcode cm in*) (* why was this bad??? *)
       let ln = promote_mcode cm in
       mkres p (Ast0.PComma(cm)) ln ln
   | Ast0.Pdots(dots) ->
@@ -563,6 +563,45 @@ and parameter_list prev = dots is_param_dots prev parameterTypeDef
 (* for export *)
 let parameter_dots x = dots is_param_dots None parameterTypeDef x
 
+(* --------------------------------------------------------------------- *)
+
+let is_define_param_dots s =
+  match Ast0.unwrap s with
+    Ast0.DPdots(_) | Ast0.DPcircles(_) -> true
+  | _ -> false
+
+let rec define_param p =
+  match Ast0.unwrap p with
+    Ast0.DParam(id) ->
+      let id = ident id in mkres p (Ast0.DParam(id)) id id
+  | Ast0.DPComma(cm) ->
+      (*let cm = bad_mcode cm in*) (* why was this bad??? *)
+      let ln = promote_mcode cm in
+      mkres p (Ast0.DPComma(cm)) ln ln
+  | Ast0.DPdots(dots) ->
+      let dots = bad_mcode dots in
+      let ln = promote_mcode dots in
+      mkres p (Ast0.DPdots(dots)) ln ln
+  | Ast0.DPcircles(dots) ->
+      let dots = bad_mcode dots in
+      let ln = promote_mcode dots in
+      mkres p (Ast0.DPcircles(dots)) ln ln
+  | Ast0.OptDParam(dp) ->
+      let res = define_param dp in
+      mkres p (Ast0.OptDParam(res)) res res
+  | Ast0.UniqueDParam(dp) ->
+      let res = define_param dp in
+      mkres p (Ast0.UniqueDParam(res)) res res
+
+let define_parameters x =
+  match Ast0.unwrap x with
+    Ast0.NoParams -> x (* no info, should be ignored *)
+  | Ast0.DParams(lp,dp,rp) ->
+      let dp = dots is_define_param_dots None define_param dp in
+      let l = promote_mcode lp in
+      let r = promote_mcode rp in
+      mkres x (Ast0.DParams(lp,dp,rp)) l r
+
 (* --------------------------------------------------------------------- *)
 (* Top-level code *)
 
@@ -628,12 +667,20 @@ let rec statement s =
 	let right = promote_to_statement body aft in
 	mkres s (Ast0.Iterator(nm,lp,args,rp,body,(Ast0.get_info right,aft)))
 	  nm right
-    | Ast0.Switch(switch,lp,exp,rp,lb,cases,rb) ->
+    | Ast0.Switch(switch,lp,exp,rp,lb,decls,cases,rb) ->
 	let exp = expression exp in
+	let decls =
+	  dots is_stm_dots (Some(promote_mcode lb))
+	    statement decls in
 	let cases =
-	  dots (function _ -> false) (Some(promote_mcode lb)) case_line cases in
+	  dots (function _ -> false)
+	    (if Ast0.undots decls = []
+	    then (Some(promote_mcode lb))
+	    else None (* not sure this is right, but not sure the case can
+			 arise either *))
+	    case_line cases in
 	mkres s
-	  (Ast0.Switch(switch,lp,exp,rp,lb,cases,rb))
+	  (Ast0.Switch(switch,lp,exp,rp,lb,decls,cases,rb))
 	  (promote_mcode switch) (promote_mcode rb)
     | Ast0.Break(br,sem) as us ->
 	mkres s us (promote_mcode br) (promote_mcode sem)
@@ -745,6 +792,7 @@ let rec statement s =
 	mkres s (Ast0.Include(inc,stm)) (promote_mcode inc) (promote_mcode stm)
     | Ast0.Define(def,id,params,body) ->
 	let id = ident id in
+	let params = define_parameters params in
 	let body = dots is_stm_dots None statement body in
 	mkres s (Ast0.Define(def,id,params,body)) (promote_mcode def) body
     | Ast0.OptStm(stm) ->
@@ -768,6 +816,14 @@ and case_line c =
       let exp = expression exp in
       let code = dots is_stm_dots (Some(promote_mcode colon)) statement code in
       mkres c (Ast0.Case(case,exp,colon,code)) (promote_mcode case) code
+  | Ast0.DisjCase(starter,case_lines,mids,ender) ->
+      let starter = bad_mcode starter in
+      let case_lines = List.map case_line case_lines in
+      let mids = List.map bad_mcode mids in
+      let ender = bad_mcode ender in
+      mkmultires c (Ast0.DisjCase(starter,case_lines,mids,ender))
+	(promote_mcode starter) (promote_mcode ender)
+	(get_all_start_info case_lines) (get_all_end_info case_lines)
   | Ast0.OptCase(case) ->
       let case = case_line case in mkres c (Ast0.OptCase(case)) case case
 
