@@ -1,27 +1,27 @@
 (*
-* Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
-* Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller
-* This file is part of Coccinelle.
-* 
-* Coccinelle is free software: you can redistribute it and/or modify
-* it under the terms of the GNU General Public License as published by
-* the Free Software Foundation, according to version 2 of the License.
-* 
-* Coccinelle is distributed in the hope that it will be useful,
-* but WITHOUT ANY WARRANTY; without even the implied warranty of
-* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-* GNU General Public License for more details.
-* 
-* You should have received a copy of the GNU General Public License
-* along with Coccinelle.  If not, see <http://www.gnu.org/licenses/>.
-* 
-* The authors reserve the right to distribute this or future versions of
-* Coccinelle under other licenses.
-*)
+ * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
+ * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
+ * This file is part of Coccinelle.
+ *
+ * Coccinelle is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, according to version 2 of the License.
+ *
+ * Coccinelle is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with Coccinelle.  If not, see <http://www.gnu.org/licenses/>.
+ *
+ * The authors reserve the right to distribute this or future versions of
+ * Coccinelle under other licenses.
+ *)
 
 
 (* create an index for each constructor *)
-(* current max is 147, but 107 is free *)
+(* current max is 147 *)
 
 (* doesn't really work - requires that identical terms with no token
 subterms (eg dots) not appear on the same line *)
@@ -76,12 +76,12 @@ let case_line_dots d =
 
 let ident i =
   match Ast0.unwrap i with
-    Ast0.Id(name) -> [10]
-  | Ast0.MetaId(name,_,_) -> [11]
-  | Ast0.MetaFunc(name,_,_) -> [12]
-  | Ast0.MetaLocalFunc(name,_,_) -> [13]
-  | Ast0.OptIdent(id) -> [14]
-  | Ast0.UniqueIdent(id) -> [15]
+      Ast0.Id(name) -> [10]
+    | Ast0.MetaId(name,_,_) -> [11]
+    | Ast0.MetaFunc(name,_,_) -> [12]
+    | Ast0.MetaLocalFunc(name,_,_) -> [13]
+    | Ast0.OptIdent(id) -> [14]
+    | Ast0.UniqueIdent(id) -> [15]
 
 let expression e =
   match Ast0.unwrap e with
@@ -181,7 +181,7 @@ let statement s =
   | Ast0.Do(d,body,whl,lp,exp,rp,sem) -> [75]
   | Ast0.For(fr,lp,e1,sem1,e2,sem2,e3,rp,body,_) -> [76]
   | Ast0.Iterator(nm,lp,args,rp,body,_) -> [142]
-  | Ast0.Switch(switch,lp,exp,rp,lb,cases,rb) -> [125]
+  | Ast0.Switch(switch,lp,exp,rp,lb,decls,cases,rb) -> [125]
   | Ast0.Break(br,sem) -> [100]
   | Ast0.Continue(cont,sem) -> [101]
   | Ast0.Label(l,dd) -> [144]
@@ -208,6 +208,7 @@ let case_line c =
   match Ast0.unwrap c with
     Ast0.Default(def,colon,code) -> [126]
   | Ast0.Case(case,exp,colon,code) -> [127]
+  | Ast0.DisjCase(_,case_lines,_,_) -> [107]
   | Ast0.OptCase(case) -> [128]
 
 let top_level t =
