@@ -1,23 +1,23 @@
 (*
-* Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
-* Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller
-* This file is part of Coccinelle.
-* 
-* Coccinelle is free software: you can redistribute it and/or modify
-* it under the terms of the GNU General Public License as published by
-* the Free Software Foundation, according to version 2 of the License.
-* 
-* Coccinelle is distributed in the hope that it will be useful,
-* but WITHOUT ANY WARRANTY; without even the implied warranty of
-* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-* GNU General Public License for more details.
-* 
-* You should have received a copy of the GNU General Public License
-* along with Coccinelle.  If not, see <http://www.gnu.org/licenses/>.
-* 
-* The authors reserve the right to distribute this or future versions of
-* Coccinelle under other licenses.
-*)
+ * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
+ * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
+ * This file is part of Coccinelle.
+ *
+ * Coccinelle is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, according to version 2 of the License.
+ *
+ * Coccinelle is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with Coccinelle.  If not, see <http://www.gnu.org/licenses/>.
+ *
+ * The authors reserve the right to distribute this or future versions of
+ * Coccinelle under other licenses.
+ *)
 
 
 (* Potential problem: offset of mcode is not updated when an iso is
@@ -44,11 +44,12 @@ type isomorphism =
 
 let strip_info =
   let mcode (term,_,_,_,_,_) =
-    (term,Ast0.NONE,Ast0.default_info(),Ast0.PLUS,ref Ast0.NoMetaPos,-1) in
+    (term,Ast0.NONE,Ast0.default_info(),Ast0.PLUS Ast.ONE,
+     ref Ast0.NoMetaPos,-1) in
   let donothing r k e =
     let x = k e in
     {(Ast0.wrap (Ast0.unwrap x)) with
-      Ast0.mcodekind = ref Ast0.PLUS;
+      Ast0.mcodekind = ref (Ast0.PLUS Ast.ONE);
       Ast0.true_if_test = x.Ast0.true_if_test} in
   V0.flat_rebuilder
     mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode mcode
@@ -71,7 +72,8 @@ let anything_equal = function
   | (Ast0.DotsCaseTag(d1),Ast0.DotsCaseTag(d2)) ->
       failwith "not a possible variable binding"
   | (Ast0.IdentTag(d1),Ast0.IdentTag(d2)) ->
-      (strip_info.VT0.rebuilder_rec_ident d1) = (strip_info.VT0.rebuilder_rec_ident d2)
+      (strip_info.VT0.rebuilder_rec_ident d1) =
+      (strip_info.VT0.rebuilder_rec_ident d2)
   | (Ast0.ExprTag(d1),Ast0.ExprTag(d2)) ->
       (strip_info.VT0.rebuilder_rec_expression d1) =
       (strip_info.VT0.rebuilder_rec_expression d2)
@@ -289,7 +291,7 @@ let rec is_pure_context s =
 	      (match Ast.unwrap s with
 		Ast.IfThen(_,_,_) -> false (* potentially dangerous *)
 	      | _ -> true)
-	  |	(_,_) -> false)
+	  | (_,_) -> false)
       | _ -> false))
 
 let is_minus e =
@@ -592,7 +594,8 @@ let match_maker checks_needed context_required whencode_allowed =
 		    expr
 		else return false
 	  | None ->
-	      add_pure_binding name pure pure_sp_code.VT0.combiner_rec_expression
+	      add_pure_binding name pure
+		pure_sp_code.VT0.combiner_rec_expression
 		(function expr -> Ast0.ExprTag expr)
 		expr
 	else return false
@@ -1043,12 +1046,14 @@ let match_maker checks_needed context_required whencode_allowed =
 		   match_dots match_expr is_elist_matcher do_elist_match
 		     argsa argsb;
 		   match_statement bodya bodyb]
-	  | (Ast0.Switch(s1,lp1,expa,rp1,lb1,casesa,rb1),
-	     Ast0.Switch(s,lp,expb,rp,lb,casesb,rb)) ->
+	  | (Ast0.Switch(s1,lp1,expa,rp1,lb1,declsa,casesa,rb1),
+	     Ast0.Switch(s,lp,expb,rp,lb,declsb,casesb,rb)) ->
 	       conjunct_many_bindings
 		 [check_mcode s1 s; check_mcode lp1 lp; check_mcode rp1 rp;
 		   check_mcode lb1 lb; check_mcode rb1 rb;
 		   match_expr expa expb;
+		   match_dots match_statement is_slist_matcher do_slist_match
+		     declsa declsb;
 		   match_dots match_case_line no_list do_nolist_match
 		     casesa casesb]
 	  | (Ast0.Break(b1,sc1),Ast0.Break(b,sc))
@@ -1162,6 +1167,8 @@ let match_maker checks_needed context_required whencode_allowed =
 	    [check_mcode ca1 ca; check_mcode c1 c; match_expr expa expb;
 	      match_dots match_statement is_slist_matcher do_slist_match
 		codea codeb]
+      | (Ast0.DisjCase(_,case_linesa,_,_),_) ->
+	  failwith "not allowed in the pattern of an isomorphism"
       |	(Ast0.OptCase(ca),Ast0.OptCase(cb)) -> match_case_line ca cb
       |	(_,Ast0.OptCase(cb)) -> match_case_line pattern cb
       |	_ -> return false
@@ -1216,7 +1223,7 @@ let make_minus =
 	    mcodekind := Ast0.MINUS(ref([],Ast0.default_token_info))
 	| _ -> failwith "make_minus: unexpected befaft")
     | Ast0.MINUS(_mc) -> () (* in the part copied from the src term *)
-    | Ast0.PLUS -> failwith "make_minus donothing: unexpected plus mcodekind"
+    | Ast0.PLUS _ -> failwith "make_minus donothing: unexpected plus mcodekind"
     | _ -> failwith "make_minus donothing: unexpected mcodekind" in
 
   let donothing r k e =
@@ -1324,12 +1331,12 @@ let make_minus =
 let rebuild_mcode start_line =
   let copy_mcodekind = function
       Ast0.CONTEXT(mc) -> Ast0.CONTEXT(ref (!mc))
-    | Ast0.MINUS(mc) -> Ast0.MINUS(ref (!mc))
-    | Ast0.MIXED(mc) -> Ast0.MIXED(ref (!mc))
-    | Ast0.PLUS ->
+    | Ast0.MINUS(mc) ->   Ast0.MINUS(ref (!mc))
+    | Ast0.MIXED(mc) ->   Ast0.MIXED(ref (!mc))
+    | Ast0.PLUS count ->
 	(* this function is used elsewhere where we need to rebuild the
 	   indices, and so we allow PLUS code as well *)
-        Ast0.PLUS in
+        Ast0.PLUS count in
 
   let mcode (term,arity,info,mcodekind,pos,adj) =
     let info =
@@ -1589,6 +1596,11 @@ let instantiate bindings mv_bindings =
 	failwith "metaexprlist not supported"
     | Ast0.Unary(exp,unop) ->
 	(match Ast0.unwrap_mcode unop with
+	  (* propagate negation only when the propagated and the encountered
+	     negation have the same transformation, when there is nothing
+	     added to the original one, and when there is nothing added to
+	     the expression into which we are doing the propagation.  This
+	     may be too conservative. *)
 	  Ast.Not ->
 	    let was_meta =
 	      (* k e doesn't change the outer structure of the term,
@@ -1599,9 +1611,7 @@ let instantiate bindings mv_bindings =
 		    Ast0.MetaExpr(name,constraints,x,form,pure) -> true
 		  | _ -> false)
 	      |	_ -> failwith "not possible" in
-	    let nomodif e =
-	      let mc = Ast0.get_mcodekind exp in
-	      match mc with
+	    let nomodif = function
 		Ast0.MINUS(x) ->
 		  (match !x with
 		    ([],_) -> true
@@ -1611,53 +1621,77 @@ let instantiate bindings mv_bindings =
 		    (Ast.NOTHING,_,_) -> true
 		  | _ -> false)
 	      |	_ -> failwith "plus not possible" in
-	    if was_meta && nomodif exp && nomodif e
+	    let same_modif newop oldop =
+	      (* only propagate ! is they have the same modification
+		 and no + code on the old one (the new one from the iso
+		 surely has no + code) *)
+	      match (newop,oldop) with
+		(Ast0.MINUS(x1),Ast0.MINUS(x2)) -> nomodif oldop
+	      |	(Ast0.CONTEXT(x1),Ast0.CONTEXT(x2)) -> nomodif oldop
+	      |	(Ast0.MIXED(x1),Ast0.MIXED(x2)) -> nomodif oldop
+	      |	_ -> false in
+	    if was_meta
 	    then
 	      let idcont x = x in
 	      let rec negate e (*for rewrapping*) res (*code to process*) k =
 		(* k accumulates parens, to keep negation outside if no
 		   propagation is possible *)
-		match Ast0.unwrap res with
-		  Ast0.Unary(e1,op) when Ast0.unwrap_mcode op = Ast.Not ->
-		    k (Ast0.rewrap e (Ast0.unwrap e1))
-		| Ast0.Edots(_,_) -> k (Ast0.rewrap e (Ast0.unwrap res))
-		| Ast0.Paren(lp,e,rp) ->
-		    negate e e
-		      (function x ->
-			k (Ast0.rewrap res (Ast0.Paren(lp,x,rp))))
-		| Ast0.Binary(e1,op,e2) ->
-		    let reb nop = Ast0.rewrap_mcode op (Ast.Logical(nop)) in
-		    let k1 x = k (Ast0.rewrap e x) in
-		    (match Ast0.unwrap_mcode op with
-		      Ast.Logical(Ast.Inf) ->
-			k1 (Ast0.Binary(e1,reb Ast.SupEq,e2))
-		    | Ast.Logical(Ast.Sup) ->
-			k1 (Ast0.Binary(e1,reb Ast.InfEq,e2))
-		    | Ast.Logical(Ast.InfEq) ->
-			k1 (Ast0.Binary(e1,reb Ast.Sup,e2))
-		    | Ast.Logical(Ast.SupEq) ->
-			k1 (Ast0.Binary(e1,reb Ast.Inf,e2))
-		    | Ast.Logical(Ast.Eq) ->
-			k1 (Ast0.Binary(e1,reb Ast.NotEq,e2))
-		    | Ast.Logical(Ast.NotEq) ->
-			k1 (Ast0.Binary(e1,reb Ast.Eq,e2))
-		    | Ast.Logical(Ast.AndLog) ->
-			k1 (Ast0.Binary(negate e1 e1 idcont,reb Ast.OrLog,
-				       negate e2 e2 idcont))
-		    | Ast.Logical(Ast.OrLog) ->
-			k1 (Ast0.Binary(negate e1 e1 idcont,reb Ast.AndLog,
-				       negate e2 e2 idcont))
-		    | _ ->
-			Ast0.rewrap e
-			  (Ast0.Unary(k res,Ast0.rewrap_mcode op Ast.Not)))
-		| Ast0.DisjExpr(lp,exps,mids,rp) ->
+		if nomodif (Ast0.get_mcodekind e)
+		then
+		  match Ast0.unwrap res with
+		    Ast0.Unary(e1,op) when Ast0.unwrap_mcode op = Ast.Not &&
+		      same_modif
+			(Ast0.get_mcode_mcodekind unop)
+			(Ast0.get_mcode_mcodekind op) ->
+			  k e1
+		  | Ast0.Edots(_,_) -> k (Ast0.rewrap e (Ast0.unwrap res))
+		  | Ast0.Paren(lp,e1,rp) ->
+		      negate e e1
+			(function x ->
+			  k (Ast0.rewrap res (Ast0.Paren(lp,x,rp))))
+		  | Ast0.Binary(e1,op,e2) when
+		      same_modif
+			(Ast0.get_mcode_mcodekind unop)
+			(Ast0.get_mcode_mcodekind op)->
+			  let reb nop =
+			    Ast0.rewrap_mcode op (Ast.Logical(nop)) in
+			  let k1 x = k (Ast0.rewrap e x) in
+			  (match Ast0.unwrap_mcode op with
+			    Ast.Logical(Ast.Inf) ->
+			      k1 (Ast0.Binary(e1,reb Ast.SupEq,e2))
+			  | Ast.Logical(Ast.Sup) ->
+			      k1 (Ast0.Binary(e1,reb Ast.InfEq,e2))
+			  | Ast.Logical(Ast.InfEq) ->
+			      k1 (Ast0.Binary(e1,reb Ast.Sup,e2))
+			  | Ast.Logical(Ast.SupEq) ->
+			      k1 (Ast0.Binary(e1,reb Ast.Inf,e2))
+			  | Ast.Logical(Ast.Eq) ->
+			      k1 (Ast0.Binary(e1,reb Ast.NotEq,e2))
+			  | Ast.Logical(Ast.NotEq) ->
+			      k1 (Ast0.Binary(e1,reb Ast.Eq,e2))
+			  | Ast.Logical(Ast.AndLog) ->
+			      k1 (Ast0.Binary(negate e1 e1 idcont,
+					      reb Ast.OrLog,
+					      negate e2 e2 idcont))
+			  | Ast.Logical(Ast.OrLog) ->
+			      k1 (Ast0.Binary(negate e1 e1 idcont,
+					      reb Ast.AndLog,
+					      negate e2 e2 idcont))
+			  | _ ->
+			      Ast0.rewrap e
+				(Ast0.Unary(k res,
+					    Ast0.rewrap_mcode op Ast.Not)))
+		  | Ast0.DisjExpr(lp,exps,mids,rp) ->
 		      (* use res because it is the transformed argument *)
-		    let exps = List.map (function e -> negate e e k) exps in
-		    Ast0.rewrap res (Ast0.DisjExpr(lp,exps,mids,rp))
-		| _ ->
+		      let exps = List.map (function e -> negate e e k) exps in
+		      Ast0.rewrap res (Ast0.DisjExpr(lp,exps,mids,rp))
+		  | _ ->
 		      (*use e, because this might be the toplevel expression*)
-		    Ast0.rewrap e
-		      (Ast0.Unary(k res,Ast0.rewrap_mcode unop Ast.Not)) in
+		      Ast0.rewrap e
+			(Ast0.Unary(k res,Ast0.rewrap_mcode unop Ast.Not))
+		else
+		  Ast0.rewrap e
+		    (Ast0.Unary(k res,Ast0.rewrap_mcode unop Ast.Not)) in
 	      negate e exp idcont
 	    else e
 	| _ -> e)
@@ -1814,33 +1848,38 @@ let merge_plus model_mcode e_mcode =
 	  let merged =
 	    match (mba,eba) with
 	      (x,Ast.NOTHING) | (Ast.NOTHING,x) -> x
-	    | (Ast.BEFORE(b1),Ast.BEFORE(b2)) -> Ast.BEFORE(b1@b2)
-	    | (Ast.BEFORE(b),Ast.AFTER(a)) -> Ast.BEFOREAFTER(b,a)
-	    | (Ast.BEFORE(b1),Ast.BEFOREAFTER(b2,a)) ->
-		Ast.BEFOREAFTER(b1@b2,a)
-	    | (Ast.AFTER(a),Ast.BEFORE(b)) -> Ast.BEFOREAFTER(b,a)
-	    | (Ast.AFTER(a1),Ast.AFTER(a2)) ->Ast.AFTER(a2@a1)
-	    | (Ast.AFTER(a1),Ast.BEFOREAFTER(b,a2)) -> Ast.BEFOREAFTER(b,a2@a1)
-	    | (Ast.BEFOREAFTER(b1,a),Ast.BEFORE(b2)) ->
-		Ast.BEFOREAFTER(b1@b2,a)
-	    | (Ast.BEFOREAFTER(b,a1),Ast.AFTER(a2)) ->
-		Ast.BEFOREAFTER(b,a2@a1)
-	    | (Ast.BEFOREAFTER(b1,a1),Ast.BEFOREAFTER(b2,a2)) ->
-		 Ast.BEFOREAFTER(b1@b2,a2@a1) in
+	    | (Ast.BEFORE(b1,it1),Ast.BEFORE(b2,it2)) ->
+		Ast.BEFORE(b1@b2,Ast.lub_count it1 it2)
+	    | (Ast.BEFORE(b,it1),Ast.AFTER(a,it2)) ->
+		Ast.BEFOREAFTER(b,a,Ast.lub_count it1 it2)
+	    | (Ast.BEFORE(b1,it1),Ast.BEFOREAFTER(b2,a,it2)) ->
+		Ast.BEFOREAFTER(b1@b2,a,Ast.lub_count it1 it2)
+	    | (Ast.AFTER(a,it1),Ast.BEFORE(b,it2)) ->
+		Ast.BEFOREAFTER(b,a,Ast.lub_count it1 it2)
+	    | (Ast.AFTER(a1,it1),Ast.AFTER(a2,it2)) ->
+		Ast.AFTER(a2@a1,Ast.lub_count it1 it2)
+	    | (Ast.AFTER(a1,it1),Ast.BEFOREAFTER(b,a2,it2)) ->
+		Ast.BEFOREAFTER(b,a2@a1,Ast.lub_count it1 it2)
+	    | (Ast.BEFOREAFTER(b1,a,it1),Ast.BEFORE(b2,it2)) ->
+		Ast.BEFOREAFTER(b1@b2,a,Ast.lub_count it1 it2)
+	    | (Ast.BEFOREAFTER(b,a1,it1),Ast.AFTER(a2,it2)) ->
+		Ast.BEFOREAFTER(b,a2@a1,Ast.lub_count it1 it2)
+	    | (Ast.BEFOREAFTER(b1,a1,it1),Ast.BEFOREAFTER(b2,a2,it2)) ->
+		 Ast.BEFOREAFTER(b1@b2,a2@a1,Ast.lub_count it1 it2) in
 	  emc := (merged,tb,ta)
       |	Ast0.MINUS(emc) ->
 	  let (anything_bef_aft,_,_) = !mc in
 	  let (anythings,t) = !emc in
 	  emc :=
 	    (match anything_bef_aft with
-	      Ast.BEFORE(b) -> (b@anythings,t)
-	    | Ast.AFTER(a) -> (anythings@a,t)
-	    | Ast.BEFOREAFTER(b,a) -> (b@anythings@a,t)
+	      Ast.BEFORE(b,_) -> (b@anythings,t)
+	    | Ast.AFTER(a,_) -> (anythings@a,t)
+	    | Ast.BEFOREAFTER(b,a,_) -> (b@anythings@a,t)
 	    | Ast.NOTHING -> (anythings,t))
       | Ast0.MIXED(_) -> failwith "how did this become mixed?"
       |	_ -> failwith "not possible 7")
   | Ast0.MIXED(_) -> failwith "not possible 8"
-  | Ast0.PLUS -> failwith "not possible 9"
+  | Ast0.PLUS _ -> failwith "not possible 9"
 
 let copy_plus printer minusify model e =
   if !Flag.sgrep_mode2
@@ -1862,7 +1901,7 @@ let copy_minus printer minusify model e =
       if !Flag.sgrep_mode2
       then e
       else failwith "not possible 8"
-  | Ast0.PLUS -> failwith "not possible 9"
+  | Ast0.PLUS _ -> failwith "not possible 9"
 
 let whencode_allowed prev_ecount prev_icount prev_dcount
     ecount icount dcount rest =
@@ -2166,11 +2205,16 @@ let transform_expr (metavars,alts,name) e =
       make_minus.VT0.rebuilder_rec_expression
       (rebuild_mcode start_line).VT0.rebuilder_rec_expression
       name Unparse_ast0.expression extra_copy_other_plus update_others in
+  let set_property model e =
+    let e = if Ast0.get_test_pos model then Ast0.set_test_exp e else e in
+    if Ast0.get_arg_exp model then Ast0.set_arg_exp e else e in
   match alts with
-    (Ast0.ExprTag(_)::_)::_ -> process do_nothing
-  | (Ast0.ArgExprTag(_)::_)::_ when Ast0.get_arg_exp e -> process do_nothing
+    (Ast0.ExprTag(_)::_)::_ ->
+      process (set_property e)
+  | (Ast0.ArgExprTag(_)::_)::_ when Ast0.get_arg_exp e ->
+      process (set_property e)
   | (Ast0.TestExprTag(_)::_)::_ when Ast0.get_test_pos e ->
-      process Ast0.set_test_exp
+      process (set_property e)
   | _ -> (0,[],e)
 
 let transform_decl (metavars,alts,name) e =
