@@ -1,23 +1,23 @@
 (*
-* Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
-* Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller
-* This file is part of Coccinelle.
-* 
-* Coccinelle is free software: you can redistribute it and/or modify
-* it under the terms of the GNU General Public License as published by
-* the Free Software Foundation, according to version 2 of the License.
-* 
-* Coccinelle is distributed in the hope that it will be useful,
-* but WITHOUT ANY WARRANTY; without even the implied warranty of
-* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-* GNU General Public License for more details.
-* 
-* You should have received a copy of the GNU General Public License
-* along with Coccinelle.  If not, see <http://www.gnu.org/licenses/>.
-* 
-* The authors reserve the right to distribute this or future versions of
-* Coccinelle under other licenses.
-*)
+ * Copyright 2005-2009, Ecole des Mines de Nantes, University of Copenhagen
+ * Yoann Padioleau, Julia Lawall, Rene Rydhof Hansen, Henrik Stuart, Gilles Muller, Nicolas Palix
+ * This file is part of Coccinelle.
+ *
+ * Coccinelle is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, according to version 2 of the License.
+ *
+ * Coccinelle is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with Coccinelle.  If not, see <http://www.gnu.org/licenses/>.
+ *
+ * The authors reserve the right to distribute this or future versions of
+ * Coccinelle under other licenses.
+ *)
 
 
 module Ast0 = Ast0_cocci
@@ -115,13 +115,13 @@ let combiner bind option_default
   and ident i =
     let k i =
       match Ast.unwrap i with
-	Ast.Id(name) -> string_mcode name
-      | Ast.MetaId(name,_,_,_) -> meta_mcode name
-      | Ast.MetaFunc(name,_,_,_) -> meta_mcode name
-      | Ast.MetaLocalFunc(name,_,_,_) -> meta_mcode name
-      | Ast.OptIdent(id) -> ident id
-      | Ast.UniqueIdent(id) -> ident id in
-    identfn all_functions k i
+	  Ast.Id(name) -> string_mcode name
+	| Ast.MetaId(name,_,_,_) -> meta_mcode name
+	| Ast.MetaFunc(name,_,_,_) -> meta_mcode name
+	| Ast.MetaLocalFunc(name,_,_,_) -> meta_mcode name
+	| Ast.OptIdent(id) -> ident id
+	| Ast.UniqueIdent(id) -> ident id in
+      identfn all_functions k i
 
   and expression e =
     let k e =
@@ -426,8 +426,9 @@ let combiner bind option_default
       | Ast.For(header,body,_) -> multibind [rule_elem header; statement body]
       | Ast.Iterator(header,body,_) ->
 	  multibind [rule_elem header; statement body]
-      |	Ast.Switch(header,lb,cases,rb) ->
+      |	Ast.Switch(header,lb,decls,cases,rb) ->
 	  multibind [rule_elem header;rule_elem lb;
+		      statement_dots decls;
 		      multibind (List.map case_line cases);
 		      rule_elem rb]
       | Ast.Atomic(re) -> rule_elem re
@@ -935,8 +936,9 @@ let rebuilder
 	    Ast.For(rule_elem header, statement body, aft)
 	| Ast.Iterator(header,body,aft) ->
 	    Ast.Iterator(rule_elem header, statement body, aft)
-	| Ast.Switch(header,lb,cases,rb) ->
+	| Ast.Switch(header,lb,decls,cases,rb) ->
 	    Ast.Switch(rule_elem header,rule_elem lb,
+		       statement_dots decls,
 		       List.map case_line cases,rule_elem rb)
 	| Ast.Atomic(re) -> Ast.Atomic(rule_elem re)
 	| Ast.Disj(stmt_dots_list) ->
